try:
    import unittest2 as unittest
except ImportError:
    import unittest

import logging
import time
import os
import datetime
import random
from wia import Wia
from wia.error import WiaError, WiaValidationError, WiaUnauthorisedError, WiaForbiddenError, WiaNotFoundError

class SpacesTest(unittest.TestCase):
    def test_spaces_create_and_retrieve(self):
        wia = Wia()
        wia.access_token_create(username=os.environ['WIA_TEST_USERNAME'],
                                               password=os.environ['WIA_TEST_PASSWORD'])
        random_name = str(datetime.date.today()) + str(random.getrandbits(128))
        space = wia.Space.create(name=random_name)
        self.assertTrue(space.name == random_name)
        space_retrieve = wia.Space.retrieve(space.id)
        self.assertTrue(space.name == space_retrieve.name)


    def test_spaces_list(self):
        wia = Wia()
        spaces = wia.Space.list()
        self.assertTrue(spaces['count'] > 0)

    def test_spaces_create_retrieve_update_list_delete_device(self):
        wia = Wia()
        spaces = wia.Space.list()
        testSpace = spaces['spaces'][0]

        testDevice = wia.Device.create(**{"name": "testDevice", "space.id": testSpace.id})
        self.assertIsInstance(testDevice, type(Wia().Device))

        retrieveDevice = wia.Device.retrieve(testDevice.id)
        self.assertTrue(retrieveDevice.name == "testDevice")

        deviceUpdated = wia.Device.update(id=retrieveDevice.id, name="testDeviceUpdated")
        self.assertTrue(deviceUpdated.name == "testDeviceUpdated")
        list_return = wia.Device.list(**{"space.id": testSpace.id})
        self.assertTrue('devices' in list_return)
        self.assertTrue(type(list_return['devices']) == list)
        self.assertTrue('count' in list_return)
        self.assertTrue(wia.Device.delete(deviceUpdated.id))


    def test_spaces_devices_list_order_desc(self):
         wia = Wia()
         spaces = wia.Space.list()
         testSpace = spaces['spaces'][0]
         list_return_desc = wia.Device.list(**{"space.id": testSpace.id, "order": "createdAt", "sort" : "desc"})
         self.assertTrue('devices' in list_return_desc)
         self.assertTrue(type(list_return_desc['devices']) == list)
         self.assertTrue('count' in list_return_desc)


    def test_spaces_devices_list_order_asc(self):
        wia = Wia()
        spaces = wia.Space.list()
        testSpace = spaces['spaces'][0]
        list_return_asc = wia.Device.list(**{"space.id": testSpace.id, "order": "createdAt", "sort": "asc"})
        self.assertTrue('devices' in list_return_asc)
        self.assertTrue(type(list_return_asc['devices']) == list)
        self.assertTrue('count' in list_return_asc)

    # # ERROR TESTS
    def test_create_device_not_authorized(self):
        wia = Wia()
        wia.access_token = os.environ['device_secret_key']
        device = wia.Device.create(name='fail', public=True)
        self.assertIsInstance(device, WiaError)
        wia.access_token = None

    def test_create_device_invalid_params(self):
        wia = Wia()
        wia.access_token = os.environ['device_secret_key']
        device = wia.Device.create(name='fail', public='Oops')
        self.assertIsInstance(device, WiaError)
        wia.access_token = None

    def test_create_device_wrong_params(self):
        wia = Wia()
        wia.access_token = os.environ['device_secret_key']
        device = wia.Device.create(name='fail', data=100)
        self.assertIsInstance(device, WiaError)
        wia.access_token = None

    def test_device_delete_unknown(self):
        wia = Wia()
        wia.access_token = os.environ['device_secret_key']
        device = wia.Device.delete('nonexisting')
        self.assertIsInstance(device, WiaError)
        wia.access_token = None

    def test_device_retrieve_unknown(self):
        wia = Wia()
        wia.access_token = os.environ['device_secret_key']
        device = wia.Device.retrieve('dev_nonexisting')
        self.assertIsInstance(device, WiaError)
        wia.access_token = None


if __name__ == '__main__':
    unittest.main()



try:
    import unittest2 as unittest
except ImportError:
    import unittest

import time
import os
import logging

from wia import Wia
from wia.error import WiaError, WiaValidationError, WiaUnauthorisedError, WiaForbiddenError, WiaNotFoundError

class InitTest(unittest.TestCase):
    def test_init_access_token(self):
        logging.info("Starting test_init_access_token")
        access_token = os.environ['WIA_TEST_DEVICE_SECRET_KEY']
        wia = Wia()
        wia.access_token = access_token
        whoami = wia.WhoAmI.retrieve()
        self.assertIsInstance(whoami, type(wia.WhoAmI))
        access_token = None
        logging.info("Finished test_init_access_token")

    def test_init_app_key_access_token(self):
        logging.info("Starting test_init_access_token")
        app_key = os.environ['WIA_TEST_APP_KEY']
        wia = Wia()
        wia.app_key = app_key
        access_token=wia.access_token_create(username=os.environ['WIA_TEST_USERNAME'],
                                password=os.environ['WIA_TEST_PASSWORD'])
        self.assertIsInstance(access_token, type(wia.AccessToken))
        access_token = None
        logging.info("Finished test_init_app_key_access_token")

    def test_init_app_key(self):
        logging.info("Starting test_init_access_token")
        app_key = os.environ['WIA_TEST_APP_KEY']
        wia = Wia()
        wia.username = os.environ['WIA_TEST_USERNAME']
        wia.password = os.environ['WIA_TEST_PASSWORD']
        wia.app_key = app_key
        whoami = wia.WhoAmI.retrieve()
        self.assertIsInstance(whoami, type(wia.WhoAmI))
        access_token = None
        logging.info("Finished test_init_app_token")


if __name__ == '__main__':
    unittest.main()



try:
    import unittest2 as unittest
except ImportError:
    import unittest

import time
import os
import logging

from wia import Wia

class StreamTest(unittest.TestCase):

    def setUp(self):
        self.wia = Wia()
        self.wia.access_token = os.environ['device_secret_key']

    def test_stream_connect(self):
        self.wia.Stream.connect()
        count = 0
        while count <= 10:
            time.sleep(0.5)
            count += 1
            if self.wia.Stream.connected:
                break
        self.assertTrue(self.wia.Stream.connected)

    def test_stream_disconnect(self):
        self.wia.Stream.disconnect()
        count = 0
        while count <= 10:
            time.sleep(0.5)
            count += 1
            if not self.wia.Stream.connected:
                break
        self.assertFalse(self.wia.Stream.connected)

if __name__ == '__main__':
    unittest2.main()



try:
    import unittest2 as unittest
except ImportError:
    import unittest

import time
import os
import requests

from wia import Wia
from wia.error import WiaError, WiaValidationError, WiaUnauthorisedError, WiaForbiddenError, WiaNotFoundError

class CommandTest(unittest.TestCase):

    def test_command_create_retrieve_update_run_delete(self):
        app_key = os.environ['WIA_TEST_APP_KEY']
        wia = Wia()
        wia.app_key = app_key
        access_token = wia.access_token_create(username=os.environ['WIA_TEST_USERNAME'],
                                               password=os.environ['WIA_TEST_PASSWORD'])
        command = wia.Command.create(**{"name": "runTest", "device.id": os.environ['device_id']})
        self.assertEqual(command.name, 'runTest')
        command = wia.Command.retrieve(command.id)
        self.assertEqual(command.name, 'runTest')
        commandUpdated = wia.Command.update(**{"id": command.id, "name": "runTestUpdated", "device.id": os.environ['device_id'], "slug": command.slug})
        self.assertEqual(commandUpdated.name, 'runTestUpdated')
        commandRun = wia.Command.run(**{"id": command.id, "name": "runTestUpdated", "device.id": os.environ['device_id'], "slug": command.slug})
        self.assertTrue(wia.Command.delete(commandUpdated.id))
        wia.access_token = None

    def test_command_list(self):
        app_key = os.environ['WIA_TEST_APP_KEY']
        wia = Wia()
        wia.app_key = app_key
        access_token = wia.access_token_create(username=os.environ['WIA_TEST_USERNAME'],
                                               password=os.environ['WIA_TEST_PASSWORD'])
        commands = wia.Command.list(**{"device.id": os.environ['device_id']})
        self.assertTrue(type(commands['commands']) == list)
        wia.access_token = None

    def test_stream_commands(self):
        app_key = os.environ['WIA_TEST_APP_KEY']
        wia = Wia()
        wia.app_key = app_key
        access_token = wia.access_token_create(username=os.environ['WIA_TEST_USERNAME'],
                                               password=os.environ['WIA_TEST_PASSWORD'])
        command = wia.Command.create(**{"name": "test-run", "device.id": os.environ['device_id']})
        wia.Stream.connect()
        count = 0
        while count <= 10:
            time.sleep(0.5)
            count += 1
            if wia.Stream.connected:
                break
        self.assertTrue(wia.Stream.connected)

        def test_run(self):
            wia.Stream.disconnect()
            count = 0
            while count <= 10:
                time.sleep(0.5)
                count += 1
                if not wia.Stream.connected:
                    break
            wia.Stream.connected = False
            wia.access_token = None

        wia.Command.subscribe(**{"device": os.environ['device_id'], "slug": command.slug, "func": test_run})
        time.sleep(2)
        json = {"slug": command.slug, 'device.id': os.environ['device_id']}
        headers = {'Authorization': 'Bearer ' + Wia().access_token}
        r = requests.post("https://api.wia.io/v1/commands/run", json=json, headers=headers)
        #wia.Command.run(**{"device": os.environ['device_id'], "slug": command.slug})
        while wia.Stream.connected:
            time.sleep(0.5)




if __name__ == '__main__':
    unittest2.main()



import time
import logging

from wia import Wia
from wia.rest_client import post, get, put, delete
from wia.error import WiaError, WiaValidationError, WiaUnauthorisedError, WiaForbiddenError, WiaNotFoundError

class WiaResource():

    @staticmethod
    def is_success(response):
        if response.status_code == 200 or response.status_code == 201:
            return True
        else:
            return False

    @staticmethod
    def error_response(response):
        if response.status_code == 400:
            return WiaValidationError(response)
        elif response.status_code == 401:
            return WiaUnauthorisedError(response)
        elif response.status_code == 403:
            return WiaForbiddenError(response)
        elif response.status_code == 404:
            return WiaNotFoundError(response)
        else:
            return WiaError(response)

class WiaResourceDelete(WiaResource):
    def __init__(self, **kwargs):
        self.id = (kwargs['id'] if 'id' in kwargs else None)
        self.deleted = (kwargs['deleted'] if 'deleted' in kwargs else None)

class Space(WiaResource):
    def __init__(self, **kwargs):
        self.id = (kwargs['id'] if 'id' in kwargs else None)
        self.name = (kwargs['name'] if 'name' in kwargs else None)
        self.createdAt = (kwargs['createdAt'] if 'createdAt' in kwargs else None)
        self.updatedAt = (kwargs['updatedAt'] if 'updatedAt' in kwargs else None)

    @classmethod
    def create(cls, **kwargs):
        """
        Creates a Space in Wia for devices
        :param kwargs:
        :return: Returns a newly created space
        """
        path = 'spaces'
        response = post(path, kwargs)
        if WiaResource.is_success(response):
            return cls(**response.json())
        else:
            return WiaResource.error_response(response)

    @classmethod
    def retrieve(cls, id):
        """
        Retrieves a Space by ID
        :param id: ID of the space
        :return: Instance of the Space object
        """
        path = 'spaces/' + id
        response = get(path)
        if WiaResource.is_success(response):
            return cls(**response.json())
        else:
            return WiaResource.error_response(response)

    @classmethod
    def update(cls, **kwargs):
        path = 'spaces/' + kwargs['id']
        dictCopy = dict(kwargs)
        del dictCopy['id']
        response = put(path, dictCopy)
        if WiaResource.is_success(response):
            return cls(**response.json())
        else:
            return WiaResource.error_response(response)

    @classmethod
    def list(cls, **kwargs):
        response = get('spaces', **kwargs)
        if WiaResource.is_success(response):
            responseJson = response.json()
            spaces = []
            for space in responseJson['spaces']:
                spaces.append(cls(**space))
            return {'spaces':spaces,'count': responseJson['count']}
        else:
            return WiaResource.error_response(response)


class Device(WiaResource):
    def __init__(self, **kwargs):
        self.id = (kwargs['id'] if 'id' in kwargs else None)
        self.name = (kwargs['name'] if 'name' in kwargs else None)
        self.events = (kwargs['events'] if 'events' in kwargs else None)
        self.location = (kwargs['location'] if 'location' in kwargs else None)
        self.createdAt = (kwargs['createdAt'] if 'createdAt' in kwargs else None)
        self.updatedAt = (kwargs['updatedAt'] if 'updatedAt' in kwargs else None)

    @classmethod
    def create(cls, **kwargs):
        path = 'devices'
        response = post(path, kwargs)
        if WiaResource.is_success(response):
            return cls(**response.json())
        else:
            return WiaResource.error_response(response)

    @classmethod
    def retrieve(cls, id):
        path = 'devices/' + id
        response = get(path)
        if WiaResource.is_success(response):
            return cls(**response.json())
        else:
            return WiaResource.error_response(response)

    @classmethod
    def update(cls, **kwargs):
        path = 'devices/' + kwargs['id']
        dictCopy = dict(kwargs)
        #del dictCopy['id']
        response = put(path, dictCopy)
        if WiaResource.is_success(response):
            return cls(**response.json())
        else:
            return WiaResource.error_response(response)

    @staticmethod
    def delete(id):
        path = 'devices/' + id
        response = delete(path)
        if WiaResource.is_success(response):
            return WiaResourceDelete(**response.json())
        else:
            return WiaResource.error_response(response)

    @classmethod
    def list(cls, **kwargs):
        response = get('devices', **kwargs)
        if WiaResource.is_success(response):
            responseJson = response.json()
            devices = []
            for device in responseJson['devices']:
                devices.append(cls(**device))
            return {'devices':devices,'count': responseJson['count']}
        else:
            return WiaResource.error_response(response)

class Event(WiaResource):
    def __init__(self, **kwargs):
        self.id = (kwargs['id'] if 'id' in kwargs else None)
        self.name = (kwargs['name'] if 'name' in kwargs else None)
        self.data = (kwargs['data'] if 'data' in kwargs else None)
        self.file = (kwargs['file'] if 'file' in kwargs else None)
        self.timestamp = (kwargs['timestamp'] if 'timestamp' in kwargs else None)

    @classmethod
    def publish(cls, **kwargs):
        path = 'events'
        if not ('file' in kwargs) and Wia().Stream.connected and Wia().client_id is not None:
            topic = 'devices/' + Wia().client_id + '/' + path + '/' + kwargs['name']
            Wia().Stream.publish(topic=topic, **kwargs)
            return cls()
        else:
            response = post(path, kwargs)
            if WiaResource.is_success(response):
                return cls(**response.json())
            else:
                return WiaResource.error_response(response)

    @staticmethod
    def subscribe(**kwargs):
        device=kwargs['device']
        topic='devices/' + device + '/events/'
        if 'name' in kwargs:
            topic += kwargs['name']
        else:
            topic += '+'
        Wia().Stream.subscribe(topic=topic, func=kwargs['func'])

    @staticmethod
    def unsubscribe(**kwargs):
        device=kwargs['device']
        topic='devices/' + device + '/events/'
        if 'name' in kwargs:
            topic += kwargs['name']
        else:
            topic += '+'
        Wia().Stream.unsubscribe(topic=topic)

    @classmethod
    def list(cls, **kwargs):
        response = get('events', **kwargs)
        if WiaResource.is_success(response):
            responseJson = response.json()
            events = []
            for event in responseJson['events']:
                events.append(cls(**event))
            return {'events':events,'count': responseJson['count']}
        else:
            return WiaResource.error_response(response)

class Location(WiaResource):
    def __init__(self, **kwargs):
        self.id = (kwargs['id'] if 'id' in kwargs else None)
        self.latitude = (kwargs['latitude'] if 'latitude' in kwargs else None)
        self.longitude = (kwargs['longitude'] if 'longitude' in kwargs else None)
        self.altitude = (kwargs['altitude'] if 'altitude' in kwargs else None)
        self.timestamp = (kwargs['timestamp'] if 'timestamp' in kwargs else None)
        self.recievedTimestamp = (kwargs['recievedTimestamp'] if 'recievedTimestamp' in kwargs else None)

    @classmethod
    def publish(cls, **kwargs):
        path = 'locations'
        if Wia().Stream.connected and Wia().client_id is not None:
            topic = 'devices/' + Wia().client_id + '/' + path
            Wia().Stream.publish(topic=topic, **kwargs)
            return cls()
        else:
            response = post(path, kwargs)
            if WiaResource.is_success(response):
                return cls(**response.json())
            else:
                return WiaResource.error_response(response)

    @staticmethod
    def subscribe(**kwargs):
        device = kwargs['device']
        topic = 'devices/' + device + '/locations'
        Wia().Stream.subscribe(topic=topic, func=kwargs['func'])

    @staticmethod
    def unsubscribe(**kwargs):
        device = kwargs['device']
        topic = 'devices/' + device + '/locations'
        Wia().Stream.unsubscribe(topic=topic)

    @classmethod
    def list(cls, **kwargs):
        response = get('locations', **kwargs)
        if WiaResource.is_success(response):
            responseJson = response.json()
            locations = []
            for location in responseJson['locations']:
                locations.append(cls(**location))
            return {'locations':locations,'count': responseJson['count']}
        else:
            return WiaResource.error_response(response)

class Command(WiaResource):
    def __init__(self, **kwargs):
        self.id = (kwargs['id'] if 'id' in kwargs else None)
        self.name = (kwargs['name'] if 'name' in kwargs else None)
        self.slug = (kwargs['slug'] if 'slug' in kwargs else None)
        self.createdAt = (kwargs['createdAt'] if 'createdAt' in kwargs else None)
        self.updatedAt = (kwargs['updatedAt'] if 'updatedAt' in kwargs else None)

    @classmethod
    def create(cls, **kwargs):
        path = 'commands'
        response = post(path, kwargs)
        if WiaResource.is_success(response):
            return cls(**response.json())
        else:
            return WiaResource.error_response(response)

    @classmethod
    def retrieve(cls, id):
        path = 'commands/' + id
        response = get(path)
        if WiaResource.is_success(response):
            return cls(**response.json())
        else:
            return WiaResource.error_response(response)

    @classmethod
    def update(cls, **kwargs):
        path = 'commands/' + kwargs['id']
        dictCopy = dict(kwargs)
        del dictCopy['id']
        response = put(path, dictCopy)
        if WiaResource.is_success(response):
            return cls(**response.json())
        else:
            return WiaResource.error_response(response)

    @staticmethod
    def delete(id):
        path = 'commands/' + id
        response = delete(path)
        if WiaResource.is_success(response):
            return WiaResourceDelete(**response.json())
        else:
            return WiaResource.error_response(response)

    @classmethod
    def list(cls, **kwargs):
        response = get('commands', **kwargs)
        if WiaResource.is_success(response):
            responseJson = response.json()
            commands = []
            for command in responseJson['commands']:
                commands.append(cls(**command))
            return {'commands':commands,'count': responseJson['count']}
        else:
            return WiaResource.error_response(response)

    @staticmethod
    def subscribe(**kwargs):
        topic='devices/' + kwargs['device'] + '/commands/' + kwargs['slug'] + '/run'
        Wia().Stream.subscribe(topic=topic, func=kwargs['func'])

    @staticmethod
    def unsubscribe(**kwargs):
        topic='devices/' + kwargs['device'] + '/commands/' + kwargs['slug'] + '/run'
        Wia().Stream.unsubscribe(topic=topic)

    @classmethod
    def run(cls, **kwargs):

        if Wia().Stream.connected and Wia().client_id is not None:
            command = 'devices/' + Wia().client_id + '/commands/' + (kwargs['slug'] or kwargs['name']) + '/run'
            Wia().Stream.run(command=command)
            return cls()
        else:
            response = post('commands/run', kwargs)
            if WiaResource.is_success(response):
                return cls(**response.json())
            else:
                return WiaResource.error_response(response)


# class Log(WiaResource):
#     def __init__(self, **kwargs):
#         self.level = (kwargs['level'] if 'level' in kwargs else None)
#         self.message = (kwargs['message'] if 'message' in kwargs else None)
#         self.data = (kwargs['data'] if 'data' in kwargs else None)
#         self.timestamp = (kwargs['timestamp'] if 'timestamp' in kwargs else None)
#
#     @classmethod
#     def publish(cls, **kwargs):
#         path = 'logs'
#         if Wia().Stream.connected and Wia().client_id is not None:
#             topic = 'devices/' + Wia().client_id + '/' + path + '/' + kwargs['level']
#             Wia().Stream.publish(topic=topic, **kwargs)
#             return cls()
#         else:
#             response = post(path, kwargs)
#             if WiaResource.is_success(response):
#                 return cls(**response.json())
#             else:
#                 return WiaResource.error_response(response)
#
#     @staticmethod
#     def subscribe(**kwargs):
#         device=kwargs['device']
#         topic='devices/' + device + '/logs/'
#         if 'level' in kwargs:
#             topic += kwargs['level']
#         else:
#             topic += '+'
#         Wia().Stream.subscribe(topic=topic, func=kwargs['func'])
#
#     @staticmethod
#     def unsubscribe(**kwargs):
#         device=kwargs['device']
#         topic='devices/' + device + '/logs/'
#         if 'level' in kwargs:
#             topic += kwargs['level']
#         else:
#             topic += '+'
#         Wia().Stream.unsubscribe(topic=topic)
#
#     @classmethod
#     def list(cls, **kwargs):
#         response = get('logs', **kwargs)
#         if WiaResource.is_success(response):
#             responseJson = response.json()
#             logs = []
#             for log in responseJson['logs']:
#                 logs.append(cls(**log))
#             return {'logs':logs,'count': responseJson['count']}
#         else:
#             return WiaResource.error_response(response)


class AccessToken(WiaResource):
    def __init__(self, **kwargs):
        self.accessToken = kwargs['token'] if 'token' in kwargs else None
        self.refreshToken = (kwargs['refreshToken'] if 'refreshToken' in kwargs else None)
        self.tokenType = (kwargs['tokenType'] if 'tokenType' in kwargs else None)
        self.expiresIn = (kwargs['expiresIn'] if 'expiresIn' in kwargs else None)
        self.scope = (kwargs['scope'] if 'scope' in kwargs else None)

    @classmethod
    def create(cls, kwargs):
        response = post('auth/token', kwargs)
        if WiaResource.is_success(response):
            return cls(**response.json())
        else:
            return WiaResource.error_response(response)

class WhoAmI(WiaResource):
    def __init__(self, **kwargs):
        self.id = (kwargs['id'] if 'id' in kwargs else None)
        self.contextData = (kwargs['contextData'] if 'contextData' in kwargs else None)
        self.scope = (kwargs['scope'] if 'scope' in kwargs else None)

    @classmethod
    def retrieve(cls):
        response = get('whoami')
        if WiaResource.is_success(response):
            return cls(**response.json())
        else:
            return WiaResource.error_response(response)



import paho.mqtt.client as mqtt
import time
import json
import re
import sys
import logging

from wia import Wia

class Stream:

    def __init__(self):
        self.connected = False
        self.subscribed = None
        self.subscribed_count = 0
        self.client = mqtt.Client()
        self.function_subscriptions = {}


    def connect(self, async=True):
        self.__init__()
        self.client.username_pw_set(Wia().access_token, ' ')
        self.client.on_connect = self.on_connect
        self.client.on_disconnect = self.on_disconnect
        self.client.on_subscribe = self.on_subscribe
        self.client.on_unsubscribe = self.on_unsubscribe
        self.client.on_message = self.on_message
        self.client.connect(Wia().stream_config['host'], Wia().stream_config['port'], 60)
        if async:
            self.client.loop_start()
        self.connected = True
        time.sleep(1)


    def disconnect(self):
        self.client.loop_stop()
        self.client.disconnect()

    def publish(self, **kwargs):
        topic = kwargs['topic']
        kwargs.pop('topic')
        self.client.publish(topic, payload=json.dumps(kwargs), qos=0, retain=False)

    def run(self, **kwargs):
        command = kwargs['command']
        kwargs.pop('command')
        self.client.publish(command, payload=json.dumps(kwargs), qos=0, retain=False)


    def subscribe(self, **kwargs):
        topic = kwargs['topic']
        self.function_subscriptions[topic] = kwargs['func']
        self.client.subscribe(topic, qos=0)



    def unsubscribe(self, **kwargs):
        topic = kwargs['topic']
        self.function_subscriptions.pop(kwargs['topic'])
        self.client.unsubscribe(topic)
        logging.debug("Unsubscribed from topic: %s", topic)


    def on_connect(self, client, userdata, flags, rc):
        self.connected = True
        logging.debug("Connected to stream")

    def on_disconnect(self, client, userdata, rc):
        self.connected = False
        logging.debug("Disconnected from stream")


    def on_subscribe(self, client, userdata, msg, granted_qos):
        self.subscribed = True
        self.subscribed_count += 1

    def on_message(self, client, userdata, msg):
        topic=re.split('/', msg.topic)
        # 1. Check for specific topic function. If exists, call
        if msg.payload:
            if msg.topic in self.function_subscriptions:
                payload = json.loads(msg.payload.decode())
                payload = dict([(str(k), v) for k, v in payload.items()])
                for k, v in payload.items():
                    if int(sys.version_info[0]) >=3:
                        if isinstance(v, bytes):
                            payload[k] = str(v)
                    else:
                        if isinstance(v, unicode):
                            payload[k] = str(v)
                self.function_subscriptions[msg.topic](payload)
        else:
            if msg.topic in self.function_subscriptions:
                self.function_subscriptions[msg.topic](None)
        # 2. Check for wildcard topic function. If exists, call
        wildcard_topic = topic[0] + "/" + topic[1] + "/" + topic[2] + "/+"
        if msg.payload:
            if wildcard_topic in self.function_subscriptions:
                if hasattr(self.function_subscriptions[wildcard_topic], '__call__'):
                    payload = json.loads(msg.payload.decode())
                    payload = dict([(str(k), v) for k, v in payload.items()])
                    for k, v in payload.items():
                        if int(sys.version_info[0]) >=3:
                            if isinstance(v, bytes):
                                payload[k] = str(v)
                        else:
                            if isinstance(v, unicode):
                                payload[k] = str(v)
                    self.function_subscriptions[wildcard_topic](payload)
        else:
            if wildcard_topic in self.function_subscriptions:
                self.function_subscriptions[wildcard_topic](None)

    def on_unsubscribe(self, client, userdata, mid):
        self.subscribed_count -= 1
        if self.subscribed_count == 0:
            self.subscribed = False




try:
    import unittest2 as unittest
except ImportError:
    import unittest

import logging
import time
import os
import sys

from wia import Wia
from wia.error import WiaError, WiaValidationError, WiaUnauthorisedError, WiaForbiddenError, WiaNotFoundError

class EventsTest(unittest.TestCase):
    def test_events_publish_rest(self):
        wia = Wia()
        wia.access_token = os.environ['device_secret_key']
        event = wia.Event.publish(name='test_event_other_rest', data=130)
        self.assertTrue(event.id is not None)
        wia.access_token = None


    def test_events_publish_file(self):
        wia = Wia()
        wia.access_token = os.environ['device_secret_key']
        dir_path = os.path.dirname(os.path.realpath(__file__))
        result = wia.Event.publish(name='test_event_other_filesud', data=1300, file=open(dir_path+'/test-file.txt', 'rb'))
        wia.access_token = None


    def test_events_publish_file_text(self):
        wia = Wia()
        wia.access_token = os.environ['device_secret_key']
        dir_path = os.path.dirname(os.path.realpath(__file__))
        result = wia.Event.publish(name='test_event_other_file', file=(dir_path+'/test-file.txt'))
        wia.access_token = None

    # ERROR TESTS
    def test_events_publish_rest_error(self):
        wia = Wia()
        wia.access_token = os.environ['device_secret_key']
        result = wia.Event.publish(abc='def')
        self.assertIsInstance(result, WiaError)
        wia.access_token = None

if __name__ == '__main__':
    unittest.main()



import requests
import logging

from wia import Wia

'''
wia_post:
    args:
        path:   string specifying url path
        kwargs: variable-length dict which can
                contain data for post request
'''
def post(path, kwargs):
    url = generate_url(path)
    headers = generate_headers(path)

    if 'file' in kwargs:
        logging.debug("Has file argument.")
        kwargsCopy = dict(kwargs)
        del kwargsCopy['file']
        if type(kwargs['file']) is str :
            with open(kwargs['file'], 'rb') as f:
                r = requests.post(url, data=kwargsCopy, headers=headers, files={'file': f})
        else:
            r = requests.post(url, data=kwargsCopy, headers=headers, files={'file': kwargs['file']})
            kwargs['file'].close()

    else:
        logging.debug("No file argument. Posting as JSON.")
        r = requests.post(url, json=kwargs, headers=headers)
    return r

'''
wia_put:
    args:
        path:   string specifying url path
        kwargs: variable-length dict which can
                contain data for put request
'''
def put(path, kwargs):
    url = generate_url(path)
    headers = generate_headers(path)
    data = kwargs
    r = requests.put(url, json=data, headers=headers)
    return r

'''
wia_get:
    args:
        path:   string specifying url path
        kwargs: variable-length dict which can
                contain query params
'''
def get(path, **kwargs):
    url = generate_url(path)
    headers = generate_headers(path)

    r = requests.get(url, headers=headers, params=kwargs)
    return r

'''
wia_delete:
    args:
        path: string specifying url path
'''
def delete(path):
    url = generate_url(path)
    headers = generate_headers(path)

    r = requests.delete(url, headers=headers)
    return r

def generate_url(path):
    if path is None:
        path = ''

    url = Wia().rest_config['protocol'] + '://' + Wia().rest_config['host'] + '/' + Wia().rest_config['basePath'] + '/' + path

    logging.debug('URL: %s', url)

    return url

def generate_headers(path):
    headers = {}

    if Wia().app_key is not None:
        headers['x-app-key'] = Wia().app_key


    if Wia().access_token is not None:
        headers['Authorization'] = 'Bearer ' + Wia().access_token


    return headers
